unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    DeviceList: TListBox;
    GroupBox2: TGroupBox;
    SubList: TListBox;
    Panel1: TPanel;
    Label1: TLabel;
    PPIsEdit: TEdit;
    SubscribeButton: TButton;
    UnsubscribeButton: TButton;
    DummyPanel: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure SubscribeButtonClick(Sender: TObject);
    procedure UnsubscribeButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  protected
    function GetOSub: Pointer;
    procedure RefreshSubList(oSub: Pointer);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOWDM;

function InStrRC(C: AnsiChar; S: AnsiString; StartIndex: Integer = MAXINT): LongWord;
var
  I: Integer;

begin
  I := Length(S);
  if StartIndex > I then StartIndex := I;
  for I := StartIndex downto 1 do
    if S[I] = C then begin
      Break;
    end
  ;

  Result := I;
end;

function NybbleToHex(N: Byte): AnsiChar;
begin
  N := N and $F;
  if N < 10 then
    Result := Char(Byte('0') + N)
  else
    Result := Char(Byte('A') - 10 + N)
  ;
end;

function StrToSpacedHex(Text: AnsiString): AnsiString;
var
  I: Integer;

begin
  Result := StringOfChar(' ', Length(Text) * 3);
  for I := 1 to Length(Text) do begin
    Result[I * 3 - 2] := NybbleToHex(Byte(Text[I]) shr 4);
    Result[I * 3 - 1] := NybbleToHex(Byte(Text[I])      );
  end;
end;

function AnsiMessageBox(Text, Caption: AnsiString; uType: UINT): Integer;
begin
  Result := MessageBoxA(Application.Handle, PAnsiChar(Text), PAnsiChar(Caption), uType);
  Text := '';
  Caption := '';
end;



const
  AM_GOTIRQ = WM_USER + $99;

type
  TSubscribePanel = class(TPanel)
  public
    constructor Create(AOwner: TComponent); override;
  protected
    procedure AMGotIRQ(var Msg: TMessage); message AM_GOTIRQ;
  end;

  TSubscriptionData = record
    CardNum: LongInt;
    PPIs: LongWord;
    COSBuf: AnsiString;
    SubscribePanel: TSubscribePanel;
  end;
  TPSubscriptionData = ^TSubscriptionData;

function StrFromSubscriptionData(pSub: TPSubscriptionData): AnsiString;
begin
  with pSub^ do
    Result := IntToStr(CardNum) + ': ' + StrToSpacedHex(COSBuf) + '(' + IntToStr(PPIs) + ') <' + IntToHex(Integer(SubscribePanel), 1) + '>'
  ;
end;

var
  Subs: array of TPSubscriptionData;

procedure TMainForm.FormCreate(Sender: TObject);
var
  NumCards, iCard: Integer;
  DeviceID, NameSize: LongWord;
  NameStr: AnsiString;

begin
  NumCards := GetNumCards;
  for iCard := 0 to NumCards-1 do begin
    NameSize := 256;
    NameStr := StringOfChar(#0, NameSize);
    if QueryCardInfo(iCard, @DeviceID, nil, @NameSize, @NameStr[1]) = 0 then begin
      NameStr := '???';
    end
    else if NameSize = 0 then begin
        NameStr := '???(' + IntToHex(DeviceID, 4) + ')';
    end
    else begin
      Dec(NameSize);
      SetLength(NameStr, NameSize);
      NameStr := NameStr + ('(' + IntToHex(DeviceID, 4) + ')');
    end;

    DeviceList.Items.Add('[' + IntToStr(iCard) + '] ' + NameStr);
  end;

  if DeviceList.Items.Count <> 0 then
    DeviceList.ItemIndex := 0
  ;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
var
  I: Integer;
  
begin
  for I := 0 to High(Subs) do
    with Subs[I]^ do
      IRQMsgUnsubscribe(CardNum, SubscribePanel.Handle)
  ;
  SetLength(Subs, 0);
end;

function TMainForm.GetOSub: Pointer;
var
  I: Integer;
  SubStr: AnsiString;

begin
  I := SubList.ItemIndex;
  if I < 0 then
    Result := nil
  else begin
    SubStr := SubList.Items[I];
    SubStr := Copy(SubStr, 1, Length(SubStr)-1);
    I := InStrRC('<', SubStr);
    if I > 0 then
      SubStr := Copy(SubStr, I+1, MAXINT)
    //else WTF mate?
    ;
    Result := Pointer(StrToInt('$' + SubStr));
  end;
end;

procedure TMainForm.RefreshSubList(oSub: Pointer);
var
  I: Integer;

begin
  SubList.Clear;
  for I := 0 to High(Subs) do begin
    SubList.Items.Add(StrFromSubscriptionData(Subs[I]));
  end;

  if oSub <> nil then begin
    for I := 0 to High(Subs) do
      if Subs[I] = oSub then begin
        SubList.ItemIndex := I;
        Break;
      end
    ;
  end;
end;

procedure TMainForm.SubscribeButtonClick(Sender: TObject);
var
  NewSub: TPSubscriptionData;
  iCard, iSub: Integer;
  oSub: Pointer;

begin
  iCard := DeviceList.ItemIndex;
  if iCard < 0 then begin
    DeviceList.ItemIndex := 0;
    Exit;
  end;

  New(NewSub);
  NewSub.CardNum := iCard;
  NewSub.PPIs := StrToInt(PPIsEdit.Text);
  SetLength(NewSub.COSBuf, 3 * NewSub.PPIs);
  NewSub.SubscribePanel := TSubscribePanel.Create(Self);
  NewSub.SubscribePanel.Parent := DummyPanel;

  with NewSub^ do begin
    if IRQMsgSubscribeData(CardNum, SubscribePanel.Handle, AM_GOTIRQ, SubscribePanel, PPIs, PByte(PAnsiChar(COSBuf))) = 0 then begin
      AnsiMessageBox('Error ' + IntToStr(GetLastError) + ' from IRQMsgSubscribeData.', 'IRQMsgSubscribeData Error', MB_ICONEXCLAMATION);
      NewSub.SubscribePanel.Free;
      Dispose(NewSub);
      Exit;
    end;
  end;

  oSub := GetOSub;

  iSub := Length(Subs);
  SetLength(Subs, iSub+1);
  Subs[iSub] := NewSub;

  RefreshSubList(oSub);
end;

procedure TMainForm.UnsubscribeButtonClick(Sender: TObject);
var
  hSub, iSub: Integer;
  oSub: Pointer;

begin
  iSub := SubList.ItemIndex;
  if iSub < 0 then begin
    SubList.ItemIndex := 0;
    Exit;
  end;

  with Subs[iSub]^ do begin
    if IRQMsgUnsubscribe(CardNum, SubscribePanel.Handle) = 0 then begin
      AnsiMessageBox('Error ' + IntToStr(GetLastError) + ' from IRQMsgSubscribeData.', 'IRQMsgSubscribeData Error', MB_ICONEXCLAMATION);
      Exit;
    end;
  end;

  oSub := GetOSub;

  hSub := High(Subs);
  if iSub < hSub then
    Move(Subs[iSub+1], Subs[iSub], SizeOf(Subs[iSub]) * (hSub - iSub))
  ;
  SetLength(Subs, hSub);

  RefreshSubList(oSub);
end;

{ TSubscribePanel }

procedure TSubscribePanel.AMGotIRQ(var Msg: TMessage);
var
  iSub: Integer;

begin
  for iSub := 0 to High(Subs) do
    if Subs[iSub].SubscribePanel = Self then begin
      MainForm.SubList.Items[iSub] := StrFromSubscriptionData(Subs[iSub]);
      Exit;
    end
  ;

  AnsiMessageBox('Context <' + IntToHex(Integer(Self), 1) + '> not found.', 'AM_GotIRQ Error', MB_ICONEXCLAMATION);
end;

constructor TSubscribePanel.Create(AOwner: TComponent);
begin
  inherited;

  Align := alTop;
end;

end.
